<?php

namespace ShineSoftware\Setefi\Model\Ui;

use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Framework\UrlInterface;
use Magento\Payment\Helper\Data as PaymentHelper;
use Magento\Framework\View\Asset\Repository as Asset;

final class MyBankConfigProvider implements ConfigProviderInterface {

    const CODE = 'setefi_mybank';

    protected $method = null;
    protected $paymentHelper;
    protected $urlBuilder;
    protected $_assetRepo;

    public function __construct(
    PaymentHelper $paymentHelper, UrlInterface $urlBuilder, Asset $assetRepo
    ) {
        $this->paymentHelper = $paymentHelper;
        $this->urlBuilder = $urlBuilder;
        $this->_assetRepo = $assetRepo;
        $this->method = $this->paymentHelper->getMethodInstance(self::CODE);
    }

    public function getConfig() {
        $redirectUrl = null;
        if ($this->method->isAvailable()) {
            $redirectUrl = $this->urlBuilder->getUrl('setefi/hosted/redirect', ['_secure' => true]);
        }
        return [
            'payment' => [
                self::CODE => [
                    'paymentAcceptanceMarkHref' => $this->getPaymentMarkWhatIsSetefiUrl(),
                    'paymentAcceptanceMarkSrc' => $this->getPaymentMarkImageUrl(),
                    'bankLink' => $this->showBankLink(),
                    'redirectUrl' => $redirectUrl,
                ]
            ]
        ];
    }

    public function getPaymentMarkImageUrl() {
        return $this->_assetRepo->getUrl("ShineSoftware_Setefi::images/logo_mybank.png");
    }

    public function getPaymentMarkWhatIsSetefiUrl() {
        return "https://www.mybank.eu/";
    }

    public function showBankLink() {
        return $this->method->getConfigData('showbanklink');
    }

}

;
